//
//  Unicode.UTF32.CodeUnit+BlockName.swift
//
//  CotEditor
//  https://coteditor.com
//
//  Created by 1024jp on 2018-08-13.
//
//  ---------------------------------------------------------------------------
//
//  © 2018-2020 1024jp
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  https://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

extension Unicode.UTF32.CodeUnit {
    
    /// Unicode block name.
    ///
    /// Based on Unicode 13.0.0 (2019-07-10)
    /// cf. <http://www.unicode.org/Public/UNIDATA/Blocks.txt>
    var blockName: String? {
        
        switch self {
            case 0x0000...0x007F: return "Basic Latin"
            case 0x0080...0x00FF: return "Latin-1 Supplement"
            case 0x0100...0x017F: return "Latin Extended-A"
            case 0x0180...0x024F: return "Latin Extended-B"
            case 0x0250...0x02AF: return "IPA Extensions"
            case 0x02B0...0x02FF: return "Spacing Modifier Letters"
            case 0x0300...0x036F: return "Combining Diacritical Marks"
            case 0x0370...0x03FF: return "Greek and Coptic"
            case 0x0400...0x04FF: return "Cyrillic"
            case 0x0500...0x052F: return "Cyrillic Supplement"
            case 0x0530...0x058F: return "Armenian"
            case 0x0590...0x05FF: return "Hebrew"
            case 0x0600...0x06FF: return "Arabic"
            case 0x0700...0x074F: return "Syriac"
            case 0x0750...0x077F: return "Arabic Supplement"
            case 0x0780...0x07BF: return "Thaana"
            case 0x07C0...0x07FF: return "NKo"
            case 0x0800...0x083F: return "Samaritan"
            case 0x0840...0x085F: return "Mandaic"
            case 0x0860...0x086F: return "Syriac Supplement"
            case 0x08A0...0x08FF: return "Arabic Extended-A"
            case 0x0900...0x097F: return "Devanagari"
            case 0x0980...0x09FF: return "Bengali"
            case 0x0A00...0x0A7F: return "Gurmukhi"
            case 0x0A80...0x0AFF: return "Gujarati"
            case 0x0B00...0x0B7F: return "Oriya"
            case 0x0B80...0x0BFF: return "Tamil"
            case 0x0C00...0x0C7F: return "Telugu"
            case 0x0C80...0x0CFF: return "Kannada"
            case 0x0D00...0x0D7F: return "Malayalam"
            case 0x0D80...0x0DFF: return "Sinhala"
            case 0x0E00...0x0E7F: return "Thai"
            case 0x0E80...0x0EFF: return "Lao"
            case 0x0F00...0x0FFF: return "Tibetan"
            case 0x1000...0x109F: return "Myanmar"
            case 0x10A0...0x10FF: return "Georgian"
            case 0x1100...0x11FF: return "Hangul Jamo"
            case 0x1200...0x137F: return "Ethiopic"
            case 0x1380...0x139F: return "Ethiopic Supplement"
            case 0x13A0...0x13FF: return "Cherokee"
            case 0x1400...0x167F: return "Unified Canadian Aboriginal Syllabics"
            case 0x1680...0x169F: return "Ogham"
            case 0x16A0...0x16FF: return "Runic"
            case 0x1700...0x171F: return "Tagalog"
            case 0x1720...0x173F: return "Hanunoo"
            case 0x1740...0x175F: return "Buhid"
            case 0x1760...0x177F: return "Tagbanwa"
            case 0x1780...0x17FF: return "Khmer"
            case 0x1800...0x18AF: return "Mongolian"
            case 0x18B0...0x18FF: return "Unified Canadian Aboriginal Syllabics Extended"
            case 0x1900...0x194F: return "Limbu"
            case 0x1950...0x197F: return "Tai Le"
            case 0x1980...0x19DF: return "New Tai Lue"
            case 0x19E0...0x19FF: return "Khmer Symbols"
            case 0x1A00...0x1A1F: return "Buginese"
            case 0x1A20...0x1AAF: return "Tai Tham"
            case 0x1AB0...0x1AFF: return "Combining Diacritical Marks Extended"
            case 0x1B00...0x1B7F: return "Balinese"
            case 0x1B80...0x1BBF: return "Sundanese"
            case 0x1BC0...0x1BFF: return "Batak"
            case 0x1C00...0x1C4F: return "Lepcha"
            case 0x1C50...0x1C7F: return "Ol Chiki"
            case 0x1C80...0x1C8F: return "Cyrillic Extended-C"
            case 0x1C90...0x1CBF: return "Georgian Extended"
            case 0x1CC0...0x1CCF: return "Sundanese Supplement"
            case 0x1CD0...0x1CFF: return "Vedic Extensions"
            case 0x1D00...0x1D7F: return "Phonetic Extensions"
            case 0x1D80...0x1DBF: return "Phonetic Extensions Supplement"
            case 0x1DC0...0x1DFF: return "Combining Diacritical Marks Supplement"
            case 0x1E00...0x1EFF: return "Latin Extended Additional"
            case 0x1F00...0x1FFF: return "Greek Extended"
            case 0x2000...0x206F: return "General Punctuation"
            case 0x2070...0x209F: return "Superscripts and Subscripts"
            case 0x20A0...0x20CF: return "Currency Symbols"
            case 0x20D0...0x20FF: return "Combining Diacritical Marks for Symbols"
            case 0x2100...0x214F: return "Letterlike Symbols"
            case 0x2150...0x218F: return "Number Forms"
            case 0x2190...0x21FF: return "Arrows"
            case 0x2200...0x22FF: return "Mathematical Operators"
            case 0x2300...0x23FF: return "Miscellaneous Technical"
            case 0x2400...0x243F: return "Control Pictures"
            case 0x2440...0x245F: return "Optical Character Recognition"
            case 0x2460...0x24FF: return "Enclosed Alphanumerics"
            case 0x2500...0x257F: return "Box Drawing"
            case 0x2580...0x259F: return "Block Elements"
            case 0x25A0...0x25FF: return "Geometric Shapes"
            case 0x2600...0x26FF: return "Miscellaneous Symbols"
            case 0x2700...0x27BF: return "Dingbats"
            case 0x27C0...0x27EF: return "Miscellaneous Mathematical Symbols-A"
            case 0x27F0...0x27FF: return "Supplemental Arrows-A"
            case 0x2800...0x28FF: return "Braille Patterns"
            case 0x2900...0x297F: return "Supplemental Arrows-B"
            case 0x2980...0x29FF: return "Miscellaneous Mathematical Symbols-B"
            case 0x2A00...0x2AFF: return "Supplemental Mathematical Operators"
            case 0x2B00...0x2BFF: return "Miscellaneous Symbols and Arrows"
            case 0x2C00...0x2C5F: return "Glagolitic"
            case 0x2C60...0x2C7F: return "Latin Extended-C"
            case 0x2C80...0x2CFF: return "Coptic"
            case 0x2D00...0x2D2F: return "Georgian Supplement"
            case 0x2D30...0x2D7F: return "Tifinagh"
            case 0x2D80...0x2DDF: return "Ethiopic Extended"
            case 0x2DE0...0x2DFF: return "Cyrillic Extended-A"
            case 0x2E00...0x2E7F: return "Supplemental Punctuation"
            case 0x2E80...0x2EFF: return "CJK Radicals Supplement"
            case 0x2F00...0x2FDF: return "Kangxi Radicals"
            case 0x2FF0...0x2FFF: return "Ideographic Description Characters"
            case 0x3000...0x303F: return "CJK Symbols and Punctuation"
            case 0x3040...0x309F: return "Hiragana"
            case 0x30A0...0x30FF: return "Katakana"
            case 0x3100...0x312F: return "Bopomofo"
            case 0x3130...0x318F: return "Hangul Compatibility Jamo"
            case 0x3190...0x319F: return "Kanbun"
            case 0x31A0...0x31BF: return "Bopomofo Extended"
            case 0x31C0...0x31EF: return "CJK Strokes"
            case 0x31F0...0x31FF: return "Katakana Phonetic Extensions"
            case 0x3200...0x32FF: return "Enclosed CJK Letters and Months"
            case 0x3300...0x33FF: return "CJK Compatibility"
            case 0x3400...0x4DBF: return "CJK Unified Ideographs Extension A"
            case 0x4DC0...0x4DFF: return "Yijing Hexagram Symbols"
            case 0x4E00...0x9FFF: return "CJK Unified Ideographs"
            case 0xA000...0xA48F: return "Yi Syllables"
            case 0xA490...0xA4CF: return "Yi Radicals"
            case 0xA4D0...0xA4FF: return "Lisu"
            case 0xA500...0xA63F: return "Vai"
            case 0xA640...0xA69F: return "Cyrillic Extended-B"
            case 0xA6A0...0xA6FF: return "Bamum"
            case 0xA700...0xA71F: return "Modifier Tone Letters"
            case 0xA720...0xA7FF: return "Latin Extended-D"
            case 0xA800...0xA82F: return "Syloti Nagri"
            case 0xA830...0xA83F: return "Common Indic Number Forms"
            case 0xA840...0xA87F: return "Phags-pa"
            case 0xA880...0xA8DF: return "Saurashtra"
            case 0xA8E0...0xA8FF: return "Devanagari Extended"
            case 0xA900...0xA92F: return "Kayah Li"
            case 0xA930...0xA95F: return "Rejang"
            case 0xA960...0xA97F: return "Hangul Jamo Extended-A"
            case 0xA980...0xA9DF: return "Javanese"
            case 0xA9E0...0xA9FF: return "Myanmar Extended-B"
            case 0xAA00...0xAA5F: return "Cham"
            case 0xAA60...0xAA7F: return "Myanmar Extended-A"
            case 0xAA80...0xAADF: return "Tai Viet"
            case 0xAAE0...0xAAFF: return "Meetei Mayek Extensions"
            case 0xAB00...0xAB2F: return "Ethiopic Extended-A"
            case 0xAB30...0xAB6F: return "Latin Extended-E"
            case 0xAB70...0xABBF: return "Cherokee Supplement"
            case 0xABC0...0xABFF: return "Meetei Mayek"
            case 0xAC00...0xD7AF: return "Hangul Syllables"
            case 0xD7B0...0xD7FF: return "Hangul Jamo Extended-B"
            case 0xD800...0xDB7F: return "High Surrogates"
            case 0xDB80...0xDBFF: return "High Private Use Surrogates"
            case 0xDC00...0xDFFF: return "Low Surrogates"
            case 0xE000...0xF8FF: return "Private Use Area"
            case 0xF900...0xFAFF: return "CJK Compatibility Ideographs"
            case 0xFB00...0xFB4F: return "Alphabetic Presentation Forms"
            case 0xFB50...0xFDFF: return "Arabic Presentation Forms-A"
            case 0xFE00...0xFE0F: return "Variation Selectors"
            case 0xFE10...0xFE1F: return "Vertical Forms"
            case 0xFE20...0xFE2F: return "Combining Half Marks"
            case 0xFE30...0xFE4F: return "CJK Compatibility Forms"
            case 0xFE50...0xFE6F: return "Small Form Variants"
            case 0xFE70...0xFEFF: return "Arabic Presentation Forms-B"
            case 0xFF00...0xFFEF: return "Halfwidth and Fullwidth Forms"
            case 0xFFF0...0xFFFF: return "Specials"
            case 0x10000...0x1007F: return "Linear B Syllabary"
            case 0x10080...0x100FF: return "Linear B Ideograms"
            case 0x10100...0x1013F: return "Aegean Numbers"
            case 0x10140...0x1018F: return "Ancient Greek Numbers"
            case 0x10190...0x101CF: return "Ancient Symbols"
            case 0x101D0...0x101FF: return "Phaistos Disc"
            case 0x10280...0x1029F: return "Lycian"
            case 0x102A0...0x102DF: return "Carian"
            case 0x102E0...0x102FF: return "Coptic Epact Numbers"
            case 0x10300...0x1032F: return "Old Italic"
            case 0x10330...0x1034F: return "Gothic"
            case 0x10350...0x1037F: return "Old Permic"
            case 0x10380...0x1039F: return "Ugaritic"
            case 0x103A0...0x103DF: return "Old Persian"
            case 0x10400...0x1044F: return "Deseret"
            case 0x10450...0x1047F: return "Shavian"
            case 0x10480...0x104AF: return "Osmanya"
            case 0x104B0...0x104FF: return "Osage"
            case 0x10500...0x1052F: return "Elbasan"
            case 0x10530...0x1056F: return "Caucasian Albanian"
            case 0x10600...0x1077F: return "Linear A"
            case 0x10800...0x1083F: return "Cypriot Syllabary"
            case 0x10840...0x1085F: return "Imperial Aramaic"
            case 0x10860...0x1087F: return "Palmyrene"
            case 0x10880...0x108AF: return "Nabataean"
            case 0x108E0...0x108FF: return "Hatran"
            case 0x10900...0x1091F: return "Phoenician"
            case 0x10920...0x1093F: return "Lydian"
            case 0x10980...0x1099F: return "Meroitic Hieroglyphs"
            case 0x109A0...0x109FF: return "Meroitic Cursive"
            case 0x10A00...0x10A5F: return "Kharoshthi"
            case 0x10A60...0x10A7F: return "Old South Arabian"
            case 0x10A80...0x10A9F: return "Old North Arabian"
            case 0x10AC0...0x10AFF: return "Manichaean"
            case 0x10B00...0x10B3F: return "Avestan"
            case 0x10B40...0x10B5F: return "Inscriptional Parthian"
            case 0x10B60...0x10B7F: return "Inscriptional Pahlavi"
            case 0x10B80...0x10BAF: return "Psalter Pahlavi"
            case 0x10C00...0x10C4F: return "Old Turkic"
            case 0x10C80...0x10CFF: return "Old Hungarian"
            case 0x10D00...0x10D3F: return "Hanifi Rohingya"
            case 0x10E60...0x10E7F: return "Rumi Numeral Symbols"
            case 0x10E80...0x10EBF: return "Yezidi"
            case 0x10F00...0x10F2F: return "Old Sogdian"
            case 0x10F30...0x10F6F: return "Sogdian"
            case 0x10FB0...0x10FDF: return "Chorasmian"
            case 0x10FE0...0x10FFF: return "Elymaic"
            case 0x11000...0x1107F: return "Brahmi"
            case 0x11080...0x110CF: return "Kaithi"
            case 0x110D0...0x110FF: return "Sora Sompeng"
            case 0x11100...0x1114F: return "Chakma"
            case 0x11150...0x1117F: return "Mahajani"
            case 0x11180...0x111DF: return "Sharada"
            case 0x111E0...0x111FF: return "Sinhala Archaic Numbers"
            case 0x11200...0x1124F: return "Khojki"
            case 0x11280...0x112AF: return "Multani"
            case 0x112B0...0x112FF: return "Khudawadi"
            case 0x11300...0x1137F: return "Grantha"
            case 0x11400...0x1147F: return "Newa"
            case 0x11480...0x114DF: return "Tirhuta"
            case 0x11580...0x115FF: return "Siddham"
            case 0x11600...0x1165F: return "Modi"
            case 0x11660...0x1167F: return "Mongolian Supplement"
            case 0x11680...0x116CF: return "Takri"
            case 0x11700...0x1173F: return "Ahom"
            case 0x11800...0x1184F: return "Dogra"
            case 0x118A0...0x118FF: return "Warang Citi"
            case 0x11900...0x1195F: return "Dives Akuru"
            case 0x119A0...0x119FF: return "Nandinagari"
            case 0x11A00...0x11A4F: return "Zanabazar Square"
            case 0x11A50...0x11AAF: return "Soyombo"
            case 0x11AC0...0x11AFF: return "Pau Cin Hau"
            case 0x11C00...0x11C6F: return "Bhaiksuki"
            case 0x11C70...0x11CBF: return "Marchen"
            case 0x11D00...0x11D5F: return "Masaram Gondi"
            case 0x11D60...0x11DAF: return "Gunjala Gondi"
            case 0x11EE0...0x11EFF: return "Makasar"
            case 0x11FB0...0x11FBF: return "Lisu Supplement"
            case 0x11FC0...0x11FFF: return "Tamil Supplement"
            case 0x12000...0x123FF: return "Cuneiform"
            case 0x12400...0x1247F: return "Cuneiform Numbers and Punctuation"
            case 0x12480...0x1254F: return "Early Dynastic Cuneiform"
            case 0x13000...0x1342F: return "Egyptian Hieroglyphs"
            case 0x13430...0x1343F: return "Egyptian Hieroglyph Format Controls"
            case 0x14400...0x1467F: return "Anatolian Hieroglyphs"
            case 0x16800...0x16A3F: return "Bamum Supplement"
            case 0x16A40...0x16A6F: return "Mro"
            case 0x16AD0...0x16AFF: return "Bassa Vah"
            case 0x16B00...0x16B8F: return "Pahawh Hmong"
            case 0x16E40...0x16E9F: return "Medefaidrin"
            case 0x16F00...0x16F9F: return "Miao"
            case 0x16FE0...0x16FFF: return "Ideographic Symbols and Punctuation"
            case 0x17000...0x187FF: return "Tangut"
            case 0x18800...0x18AFF: return "Tangut Components"
            case 0x18B00...0x18CFF: return "Khitan Small Script"
            case 0x18D00...0x18D8F: return "Tangut Supplement"
            case 0x1B000...0x1B0FF: return "Kana Supplement"
            case 0x1B100...0x1B12F: return "Kana Extended-A"
            case 0x1B130...0x1B16F: return "Small Kana Extension"
            case 0x1B170...0x1B2FF: return "Nushu"
            case 0x1BC00...0x1BC9F: return "Duployan"
            case 0x1BCA0...0x1BCAF: return "Shorthand Format Controls"
            case 0x1D000...0x1D0FF: return "Byzantine Musical Symbols"
            case 0x1D100...0x1D1FF: return "Musical Symbols"
            case 0x1D200...0x1D24F: return "Ancient Greek Musical Notation"
            case 0x1D2E0...0x1D2FF: return "Mayan Numerals"
            case 0x1D300...0x1D35F: return "Tai Xuan Jing Symbols"
            case 0x1D360...0x1D37F: return "Counting Rod Numerals"
            case 0x1D400...0x1D7FF: return "Mathematical Alphanumeric Symbols"
            case 0x1D800...0x1DAAF: return "Sutton SignWriting"
            case 0x1E000...0x1E02F: return "Glagolitic Supplement"
            case 0x1E100...0x1E14F: return "Nyiakeng Puachue Hmong"
            case 0x1E2C0...0x1E2FF: return "Wancho"
            case 0x1E800...0x1E8DF: return "Mende Kikakui"
            case 0x1E900...0x1E95F: return "Adlam"
            case 0x1EC70...0x1ECBF: return "Indic Siyaq Numbers"
            case 0x1ED00...0x1ED4F: return "Ottoman Siyaq Numbers"
            case 0x1EE00...0x1EEFF: return "Arabic Mathematical Alphabetic Symbols"
            case 0x1F000...0x1F02F: return "Mahjong Tiles"
            case 0x1F030...0x1F09F: return "Domino Tiles"
            case 0x1F0A0...0x1F0FF: return "Playing Cards"
            case 0x1F100...0x1F1FF: return "Enclosed Alphanumeric Supplement"
            case 0x1F200...0x1F2FF: return "Enclosed Ideographic Supplement"
            case 0x1F300...0x1F5FF: return "Miscellaneous Symbols and Pictographs"
            case 0x1F600...0x1F64F: return "Emoticons"
            case 0x1F650...0x1F67F: return "Ornamental Dingbats"
            case 0x1F680...0x1F6FF: return "Transport and Map Symbols"
            case 0x1F700...0x1F77F: return "Alchemical Symbols"
            case 0x1F780...0x1F7FF: return "Geometric Shapes Extended"
            case 0x1F800...0x1F8FF: return "Supplemental Arrows-C"
            case 0x1F900...0x1F9FF: return "Supplemental Symbols and Pictographs"
            case 0x1FA00...0x1FA6F: return "Chess Symbols"
            case 0x1FA70...0x1FAFF: return "Symbols and Pictographs Extended-A"
            case 0x1FB00...0x1FBFF: return "Symbols for Legacy Computing"
            case 0x20000...0x2A6DF: return "CJK Unified Ideographs Extension B"
            case 0x2A700...0x2B73F: return "CJK Unified Ideographs Extension C"
            case 0x2B740...0x2B81F: return "CJK Unified Ideographs Extension D"
            case 0x2B820...0x2CEAF: return "CJK Unified Ideographs Extension E"
            case 0x2CEB0...0x2EBEF: return "CJK Unified Ideographs Extension F"
            case 0x2F800...0x2FA1F: return "CJK Compatibility Ideographs Supplement"
            case 0x30000...0x3134F: return "CJK Unified Ideographs Extension G"
            case 0xE0000...0xE007F: return "Tags"
            case 0xE0100...0xE01EF: return "Variation Selectors Supplement"
            case 0xF0000...0xFFFFF: return "Supplementary Private Use Area-A"
            case 0x100000...0x10FFFF: return "Supplementary Private Use Area-B"
            
            default: return nil
        }
    }
    
}
